﻿Imports System.Collections.ObjectModel
Imports System.Xml

''' <summary>
''' To load and save files. This is done when starting and closing application.
''' </summary>
''' <remarks>ToDo: Include any file > load / save buttons if needed</remarks>
Public Class IOHandler
    Public File As IO.FileInfo

    Public Function LoadData() As List(Of IObject)
        Dim reader As New XmlTextReader(File.FullName)
        Dim res As New List(Of IObject)


        Try
            While reader.Read
                Select Case reader.NodeType
                    Case XmlNodeType.Element
                        Select Case reader.Name
                            Case "SceneInfo"
                                
                            Case "IObject"
                                If reader.AttributeCount > 0 Then
                                    Dim Obj As IObject = Nothing

                                    reader.MoveToNextAttribute()
                                    Dim Title As String = reader.Value
                                    reader.MoveToNextAttribute()
                                    Dim Location As String = reader.Value
                                    reader.MoveToNextAttribute()
                                    Dim Index As String = reader.Value
                                    reader.MoveToNextAttribute()
                                    Dim NeededNextObject As String = reader.Value

                                    If Title.Contains(".") Then Title = Title.Substring(Title.LastIndexOf(".") + 1)
                                    Obj = Me.StringToIObject(Title)
                                    If Obj IsNot Nothing Then
                                        Try
                                            Obj.Index = Integer.Parse(Index)
                                            Obj.Location = Point.Parse(Location)
                                            If NeededNextObject.Contains(".") Then NeededNextObject = NeededNextObject.Substring(NeededNextObject.LastIndexOf(".") + 1)
                                            Obj.NeededNextObject = StringToType(NeededNextObject)
                                        Catch ex As Exception

                                        End Try
                                        res.Add(Obj)
                                    End If

                                End If
                        End Select
                End Select
            End While
        Catch ex As Exception

        End Try


        reader.Close()

        Return res
    End Function



    Public Function StringToType(ByVal value As String) As Type
        Dim res As Type = Nothing
        Select Case value
            Case "clsBank" : res = GetType(clsBank)
            Case "clsLove" : res = GetType(clsLove)
            Case "clsPostOffice" : res = GetType(clsPostOffice)
            Case "clsShopping" : res = GetType(clsShopping)
            Case "clsToilete" : res = GetType(clsToilete)

        End Select
        Return res
    End Function


    Public Function StringToIObject(ByVal value As String) As IObject
        Dim res As IObject = Nothing
        Select Case value
            Case "Start" : res = New clsStart
            Case "Ende" : res = New clsEnd
            Case "Bank" : res = New clsBank
            Case "Liebe" : res = New clsLove
            Case "Postamt" : res = New clsPostOffice
            Case "Einkaufszentrum" : res = New clsShopping
            Case "Toilette" : res = New clsToilete

        End Select
        Return res
    End Function

    Public Sub SaveData(ByVal p As List(Of IObject))
        Dim writer As New XmlTextWriter(File.FullName, System.Text.Encoding.UTF8)

        writer.WriteStartDocument()
        writer.WriteStartElement("SceneInfo")
        writer.WriteAttributeString("Title", My.Computer.Name)

        writer.WriteStartElement("Content")
        For Each item As IObject In p
            writer.WriteStartElement("IObject")

            writer.WriteAttributeString("Title", item.Title)
            writer.WriteAttributeString("Location", item.Location.ToString)
            writer.WriteAttributeString("Index", item.Index)
            If item.NeededNextObject Is Nothing Then
                writer.WriteAttributeString("NeededNextObject", "NOTHING")
            Else
                writer.WriteAttributeString("NeededNextObject", item.NeededNextObject.ToString)
            End If

            writer.WriteEndElement() '/IObject
        Next
        writer.WriteEndElement() '/Content
        writer.WriteEndElement() '/SceneInfo
        writer.WriteEndDocument() '/document

        writer.Flush()
        writer.Close()

    End Sub







    ''' <summary>
    ''' Diese Funktion serialisiert ein Objekt und speichert es als XML-Format Datei
    ''' </summary>
    ''' <typeparam name="T">Der Typ des Objekts</typeparam>
    ''' <param name="Obj">Das zu serialisierende Objekt</param>
    ''' <param name="FileName">Die XML Datei in der das Ergebnis gespeichert werden soll</param>
    Public Sub XMLSerialize(Of T)(ByVal Obj As Object, ByVal FileName As String)
        Dim XML As New System.Xml.Serialization.XmlSerializer(GetType(T))
        Dim FS As New System.IO.FileStream(FileName, System.IO.FileMode.Create)
        XML.Serialize(FS, Obj)
        FS.Close()
    End Sub

    ''' <summary>
    ''' Diese Funktion deserialsiert ein Objekt und speichert es als XML-Format Datei
    ''' </summary>
    ''' <typeparam name="T">Der Typ des Objekts</typeparam>
    ''' <param name="FileName">Die XML Datei in der das Ergebnis gespeichert werden soll</param>
    Public Function XMLDeserialize(Of T)(ByVal FileName As String) As T
        Dim Obj As T
        Dim XML As New System.Xml.Serialization.XmlSerializer(GetType(T))
        Dim FS As New System.IO.FileStream(FileName, System.IO.FileMode.Create)
        Obj = XML.Deserialize(FS)
        FS.Close()
    End Function

    'how to call: 
    'Private Sub Window_Closing(sender As System.Object, e As System.ComponentModel.CancelEventArgs)
    '    Call XMLSerialize(Of IObject)(Application.WorkList, File.FullName)
    'End Sub

End Class
